from django.utils.translation import gettext_lazy as _

from mayan.apps.navigation.classes import Link

from ..icons import (
    icon_document_file_page_navigation_first,
    icon_document_file_page_navigation_last,
    icon_document_file_page_navigation_next,
    icon_document_file_page_navigation_previous,
    icon_document_file_page_return_to_document,
    icon_document_file_page_return_to_document_file,
    icon_document_file_page_return_to_document_file_page_list,
    icon_document_file_page_rotate_left,
    icon_document_file_page_rotate_right, icon_document_file_page_zoom_in,
    icon_document_file_page_zoom_out, icon_document_file_page_detail,
    icon_document_file_page_detail_reset, icon_document_file_page_list
)
from ..permissions import (
    permission_document_file_view, permission_document_view
)
from ..settings import setting_zoom_max_level, setting_zoom_min_level


def is_first_page(context):
    return context['resolved_object'].siblings.first() == context['resolved_object']


def is_last_page(context):
    return context['resolved_object'].siblings.last() == context['resolved_object']


def is_max_zoom(context):
    return context['zoom'] >= setting_zoom_max_level.value


def is_min_zoom(context):
    return context['zoom'] <= setting_zoom_min_level.value


link_document_file_page_navigation_first = Link(
    args='resolved_object.pk', conditional_disable=is_first_page,
    icon=icon_document_file_page_navigation_first,
    keep_query=True, permission=permission_document_file_view,
    text=_(message='First page'),
    view='documents:document_file_page_navigation_first'
)
link_document_file_page_navigation_last = Link(
    args='resolved_object.pk', conditional_disable=is_last_page,
    icon=icon_document_file_page_navigation_last,
    keep_query=True, text=_(message='Last page'),
    permission=permission_document_file_view,
    view='documents:document_file_page_navigation_last'
)
link_document_file_page_navigation_previous = Link(
    args='resolved_object.pk', conditional_disable=is_first_page,
    icon=icon_document_file_page_navigation_previous,
    keep_query=True, permission=permission_document_file_view,
    text=_(message='Previous page'),
    view='documents:document_file_page_navigation_previous'
)
link_document_file_page_navigation_next = Link(
    args='resolved_object.pk', conditional_disable=is_last_page,
    icon=icon_document_file_page_navigation_next,
    keep_query=True, text=_(message='Next page'),
    permission=permission_document_file_view,
    view='documents:document_file_page_navigation_next'
)
link_document_file_page_return_to_document = Link(
    args='resolved_object.document_file.document.pk',
    icon=icon_document_file_page_return_to_document,
    permission=permission_document_view, text=_(message='Document'),
    view='documents:document_preview'
)
link_document_file_page_return_to_document_file = Link(
    args='resolved_object.document_file.pk',
    icon=icon_document_file_page_return_to_document_file,
    permission=permission_document_file_view, text=_(message='Document file'),
    view='documents:document_file_preview'
)
link_document_file_page_return_to_document_file_page_list = Link(
    args='resolved_object.document_file.pk',
    icon=icon_document_file_page_return_to_document_file_page_list,
    permission=permission_document_file_view,
    text=_(message='Document file pages'), view='documents:document_file_page_list'
)
link_document_file_page_rotate_left = Link(
    args='resolved_object.pk', icon=icon_document_file_page_rotate_left,
    keep_query=True, permission=permission_document_file_view,
    text=_(message='Rotate left'), view='documents:document_file_page_rotate_left'
)
link_document_file_page_rotate_right = Link(
    args='resolved_object.pk', icon=icon_document_file_page_rotate_right,
    keep_query=True, permission=permission_document_file_view,
    text=_(message='Rotate right'), view='documents:document_file_page_rotate_right'
)
link_document_file_page_view = Link(
    args='resolved_object.pk', icon=icon_document_file_page_detail,
    permission=permission_document_file_view, text=_(message='Page image'),
    view='documents:document_file_page_view'
)
link_document_file_page_view_reset = Link(
    args='resolved_object.pk', icon=icon_document_file_page_detail_reset,
    permission=permission_document_file_view, text=_(message='Reset view'),
    view='documents:document_file_page_view_reset'
)
link_document_file_page_zoom_in = Link(
    args='resolved_object.pk', conditional_disable=is_max_zoom,
    icon=icon_document_file_page_zoom_in, keep_query=True,
    permission=permission_document_file_view, text=_(message='Zoom in'),
    view='documents:document_file_page_zoom_in'
)
link_document_file_page_zoom_out = Link(
    args='resolved_object.pk', conditional_disable=is_min_zoom,
    icon=icon_document_file_page_zoom_out, keep_query=True,
    permission=permission_document_file_view, text=_(message='Zoom out'),
    view='documents:document_file_page_zoom_out'
)
link_document_file_page_list = Link(
    args='resolved_object.pk', icon=icon_document_file_page_list,
    permission=permission_document_file_view, text=_(message='Pages'),
    view='documents:document_file_page_list'
)



import importlib
import logging

from mayan.apps.testing.tests.base import BaseTestCase

from mayan.apps.storage import storages
from ..classes import DefinedStorage
from ..literals import STORAGE_NAME_SHARED_UPLOADED_FILE
from ..settings import setting_shared_storage_arguments


class CommonStorageSettingsTestCase(BaseTestCase):
    def tearDown(self):
        super().tearDown()
        importlib.reload(storages)

    def test_setting_shared_storage_arguments_invalid_value(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(
                setting_shared_storage_arguments.global_name
            ), value='invalid_value'
        )
        self.test_case_silenced_logger_new_level = logging.FATAL + 10
        self._silence_logger(name='mayan.apps.storage.classes')

        with self.assertRaises(expected_exception=TypeError) as assertion:
            importlib.reload(storages)
            defined_storage = DefinedStorage.get(
                name=STORAGE_NAME_SHARED_UPLOADED_FILE
            )
            defined_storage.get_storage_instance()

        self.assertTrue(
            'Unable to initialize' in str(assertion.exception)
        )
        self.assertTrue(
            'shared uploaded' in str(assertion.exception)
        )



from django.conf import settings
from django.urls import include, re_path

from .base import *  # NOQA

if 'rosetta' in settings.INSTALLED_APPS:
    try:
        import rosetta  # NOQA
    except ImportError:
        pass
    else:
        urlpatterns += [  # NOQA
            re_path(
                route=r'^rosetta/', view=include('rosetta.urls'),
                name='rosetta'
            )
        ]

if 'debug_toolbar' in settings.INSTALLED_APPS:
    try:
        import debug_toolbar
    except ImportError:
        pass
    else:
        urlpatterns += [  # NOQA
            re_path(
                route=r'^__debug__/', view=include(debug_toolbar.urls)
            )
        ]

if 'silk' in settings.INSTALLED_APPS:
    try:
        import silk
    except ImportError:
        pass
    else:
        urlpatterns += [  # NOQA
            re_path(
                route=r'^silk/', view=include('silk.urls', namespace='silk')
            )
        ]



from mayan.apps.documents.events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_version_created,
    event_document_version_edited, event_document_version_page_created
)
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.tests.base import GenericDocumentTestCase

from .literals import (
    TEST_EMAIL_NO_CONTENT_TYPE, TEST_EMAIL_NO_CONTENT_TYPE_DATE,
    TEST_EMAIL_NO_CONTENT_TYPE_DEVELIVERED_TO,
    TEST_EMAIL_NO_CONTENT_TYPE_FROM, TEST_EMAIL_NO_CONTENT_TYPE_MESSAGE_ID,
    TEST_EMAIL_NO_CONTENT_TYPE_RECEIVED, TEST_EMAIL_NO_CONTENT_TYPE_STRING,
    TEST_EMAIL_NO_CONTENT_TYPE_SUBJECT, TEST_EMAIL_NO_CONTENT_TYPE_TO
)
from .mixins import EmailSourceTestMixin


class EmailSourceBackendDocumentUploadSourceMetadataTestCase(
    EmailSourceTestMixin, GenericDocumentTestCase
):
    _test_source_create_auto = False
    auto_upload_test_document = False

    def test_no_content_type_source_metadata(self):
        self._test_source_content = TEST_EMAIL_NO_CONTENT_TYPE
        self._test_source_create(
            extra_data={
                'store_body': True
            }
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertTrue(
            TEST_EMAIL_NO_CONTENT_TYPE_STRING in test_document_file.open().read()
        )

        self.assertEqual(test_document_file.source_metadata.count(), 8)

        self.assertEqual(
            test_document_file.source_metadata.get(key='email_date').value,
            TEST_EMAIL_NO_CONTENT_TYPE_DATE
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_delivered_to').value,
            TEST_EMAIL_NO_CONTENT_TYPE_DEVELIVERED_TO
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_from').value,
            TEST_EMAIL_NO_CONTENT_TYPE_FROM
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_message_id').value,
            TEST_EMAIL_NO_CONTENT_TYPE_MESSAGE_ID
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_received').value,
            TEST_EMAIL_NO_CONTENT_TYPE_RECEIVED
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_subject').value,
            TEST_EMAIL_NO_CONTENT_TYPE_SUBJECT
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='email_to').value,
            TEST_EMAIL_NO_CONTENT_TYPE_TO
        )
        self.assertEqual(
            test_document_file.source_metadata.get(key='source_id').value,
            str(self._test_source.pk)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 6)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, test_document)
        self.assertEqual(events[5].actor, test_document_version)
        self.assertEqual(events[5].target, test_document_version)
        self.assertEqual(events[5].verb, event_document_version_edited.id)



import hashlib

from mayan.apps.django_gpg.events import event_key_created
from mayan.apps.django_gpg.tests.literals import (
    TEST_KEY_PRIVATE_PASSPHRASE, TEST_KEY_PUBLIC_ID
)
from mayan.apps.django_gpg.tests.mixins import KeyTestMixin
from mayan.apps.documents.models.document_file_models import DocumentFile
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import (
    TEST_FILE_PDF_PATH, TEST_FILE_SMALL_PATH
)

from ..events import (
    event_detached_signature_created, event_detached_signature_deleted,
    event_detached_signature_uploaded
)
from ..models import DetachedSignature, EmbeddedSignature
from ..tasks import task_verify_missing_embedded_signature

from .literals import TEST_SIGNED_DOCUMENT_PATH, TEST_SIGNATURE_ID
from .mixins import DetachedSignatureTestMixin


class DetachedSignaturesTestCase(
    KeyTestMixin, DetachedSignatureTestMixin, GenericDocumentTestCase
):
    auto_upload_test_document = False

    def test_detached_signature_create(self):
        self._create_test_key_private()

        self._upload_test_document()

        self._clear_events()

        test_detached_signature = DetachedSignature.objects.sign_document_file(
            document_file=self._test_document.file_latest,
            key=self._test_key_private,
            passphrase=TEST_KEY_PRIVATE_PASSPHRASE
        )

        self.assertEqual(DetachedSignature.objects.count(), 1)
        self.assertTrue(
            test_detached_signature.signature_file.file is not None
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, test_detached_signature)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_detached_signature_created.id)

    def test_detached_signature_delete(self):
        self._create_test_key_private()

        self._upload_test_document()
        self._upload_test_detached_signature()

        self._clear_events()

        self._test_signature.delete()

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_detached_signature_deleted.id)

    def test_detached_signature_upload_no_key(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self._clear_events()

        self._upload_test_detached_signature()

        self.assertEqual(DetachedSignature.objects.count(), 1)

        self.assertEqual(
            self._test_signature.document_file,
            self._test_document.file_latest
        )
        self.assertEqual(self._test_signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(self._test_signature.public_key_fingerprint, None)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_signature)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(
            events[0].verb, event_detached_signature_uploaded.id
        )

    def test_detached_signature_upload_with_key(self):
        self._create_test_key_public()
        self._test_document_path = TEST_FILE_PDF_PATH
        self._upload_test_document()

        self._clear_events()

        self._upload_test_detached_signature()

        self.assertEqual(DetachedSignature.objects.count(), 1)

        self.assertEqual(
            self._test_signature.document_file,
            self._test_document.file_latest
        )
        self.assertEqual(self._test_signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(
            self._test_signature.public_key_fingerprint,
            self._test_key_public.fingerprint
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_signature)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(
            events[0].verb, event_detached_signature_uploaded.id
        )

    def test_detached_signature_upload_post_key_verify(self):
        self._test_document_path = TEST_FILE_PDF_PATH
        self._upload_test_document()

        self._clear_events()

        self._upload_test_detached_signature()

        self.assertEqual(DetachedSignature.objects.count(), 1)

        self.assertEqual(
            self._test_signature.document_file,
            self._test_document.file_latest
        )
        self.assertEqual(self._test_signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(self._test_signature.public_key_fingerprint, None)

        self._create_test_key_public()

        signature = DetachedSignature.objects.first()

        self.assertEqual(
            signature.public_key_fingerprint, self._test_key_public.fingerprint
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_signature)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(
            events[0].verb, event_detached_signature_uploaded.id
        )

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_key_public)
        self.assertEqual(events[1].target, self._test_key_public)
        self.assertEqual(events[1].verb, event_key_created.id)

    def test_detached_signature_upload_post_no_key_verify(self):
        self._create_test_key_public()
        self._test_document_path = TEST_FILE_PDF_PATH
        self._upload_test_document()

        self._clear_events()

        self._upload_test_detached_signature()

        self.assertEqual(DetachedSignature.objects.count(), 1)

        self.assertEqual(
            self._test_signature.document_file,
            self._test_document.file_latest
        )
        self.assertEqual(self._test_signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(
            self._test_signature.public_key_fingerprint,
            self._test_key_public.fingerprint
        )

        self._test_key_public.delete()

        signature = DetachedSignature.objects.first()

        self.assertEqual(signature.public_key_fingerprint, None)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_signature)
        self.assertEqual(events[0].actor, self._test_document_file)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(
            events[0].verb, event_detached_signature_uploaded.id
        )


class DocumentSignaturesTestCase(
    DetachedSignatureTestMixin, GenericDocumentTestCase
):
    auto_upload_test_document = False

    def test_unsigned_document_file_method(self):
        TEST_UNSIGNED_DOCUMENT_COUNT = 2
        TEST_SIGNED_DOCUMENT_COUNT = 2

        self._test_document_path = TEST_FILE_SMALL_PATH
        for count in range(TEST_UNSIGNED_DOCUMENT_COUNT):
            self._upload_test_document()

        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        for count in range(TEST_SIGNED_DOCUMENT_COUNT):
            self._upload_test_document()

        self.assertEqual(
            EmbeddedSignature.objects.unsigned_document_files().count(),
            TEST_UNSIGNED_DOCUMENT_COUNT
        )

    def test_method_get_absolute_url(self):
        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self._upload_test_document()

        EmbeddedSignature.objects.first().get_absolute_url()


class EmbeddedSignaturesTestCase(
    KeyTestMixin, DetachedSignatureTestMixin, GenericDocumentTestCase
):
    auto_upload_test_document = False

    def test_embedded_signature_no_key(self):
        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self._upload_test_document()

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        signature = EmbeddedSignature.objects.first()
        self.assertEqual(
            signature.document_file, self._test_document.file_latest
        )
        self.assertEqual(signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(signature.signature_id, None)

    def test_embedded_signature_post_key_verify(self):
        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self._upload_test_document()

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        signature = EmbeddedSignature.objects.first()
        self.assertEqual(
            signature.document_file, self._test_document.file_latest
        )
        self.assertEqual(signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(signature.signature_id, None)

        self._create_test_key_public()

        signature = EmbeddedSignature.objects.first()

        self.assertEqual(signature.signature_id, TEST_SIGNATURE_ID)

    def test_embedded_signature_post_no_key_verify(self):
        self._create_test_key_public()
        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self._upload_test_document()

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        signature = EmbeddedSignature.objects.first()

        self.assertEqual(
            signature.document_file, self._test_document.file_latest
        )
        self.assertEqual(signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(signature.signature_id, TEST_SIGNATURE_ID)

        self._test_key_public.delete()

        signature = EmbeddedSignature.objects.first()

        self.assertEqual(signature.signature_id, None)

    def test_embedded_signature_with_key(self):
        self._create_test_key_public()
        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self._upload_test_document()

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        signature = EmbeddedSignature.objects.first()

        self.assertEqual(
            signature.document_file,
            self._test_document.file_latest
        )
        self.assertEqual(signature.key_id, TEST_KEY_PUBLIC_ID)
        self.assertEqual(
            signature.public_key_fingerprint,
            self._test_key_public.fingerprint
        )
        self.assertEqual(signature.signature_id, TEST_SIGNATURE_ID)

    def test_task_verify_missing_embedded_signature(self):
        # Silence converter logging
        self._silence_logger(name='mayan.apps.converter.backends')

        old_hooks = DocumentFile._post_save_hooks

        DocumentFile._post_save_hooks = {}

        TEST_UNSIGNED_DOCUMENT_COUNT = 2
        TEST_SIGNED_DOCUMENT_COUNT = 2

        self._test_document_path = TEST_FILE_SMALL_PATH
        for count in range(TEST_UNSIGNED_DOCUMENT_COUNT):
            self._upload_test_document()

        self._test_document_path = TEST_SIGNED_DOCUMENT_PATH
        for count in range(TEST_SIGNED_DOCUMENT_COUNT):
            self._upload_test_document()

        self.assertEqual(
            EmbeddedSignature.objects.unsigned_document_files().count(),
            TEST_UNSIGNED_DOCUMENT_COUNT + TEST_SIGNED_DOCUMENT_COUNT
        )

        DocumentFile._post_save_hooks = old_hooks

        task_verify_missing_embedded_signature.apply_async()

        self.assertEqual(
            EmbeddedSignature.objects.unsigned_document_files().count(),
            TEST_UNSIGNED_DOCUMENT_COUNT
        )

    def test_embedded_signing(self):
        self._create_test_key_private()

        self._test_document_path = TEST_FILE_PDF_PATH
        self._upload_test_document()
        test_document_file_count = self._test_document.files.count()

        with self._test_document.file_latest.open() as file_object:
            file_object.seek(0, 2)
            original_size = file_object.tell()
            file_object.seek(0)
            original_hash = hashlib.sha256(
                string=file_object.read()
            ).hexdigest()

        signature = EmbeddedSignature.objects.sign_document_file(
            document_file=self._test_document.file_latest,
            key=self._test_key_private,
            passphrase=TEST_KEY_PRIVATE_PASSPHRASE
        )

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        with signature.document_file.open() as file_object:
            file_object.seek(0, 2)
            new_size = file_object.tell()
            file_object.seek(0)
            new_hash = hashlib.sha256(
                string=file_object.read()
            ).hexdigest()

        self.assertEqual(original_size, new_size)
        self.assertEqual(original_hash, new_hash)
        self.assertEqual(
            self._test_document.files.count(), test_document_file_count + 1
        )

        test_document_file_signed = self._test_document.files.last()
        self.assertNotEqual(
            self._test_document_file.checksum,
            test_document_file_signed.checksum
        )
        self.assertEqual(
            self._test_document_file.mimetype,
            test_document_file_signed.mimetype
        )
        self.assertEqual(
            self._test_document_file.pages.count(),
            test_document_file_signed.pages.count()
        )
        self.assertNotEqual(
            self._test_document_file.size,
            test_document_file_signed.size
        )

        with test_document_file_signed.get_download_file_object() as file_object:
            test_document_file_signed_download_file_checksum = hashlib.sha256(
                string=file_object.read()
            ).hexdigest()

        self.assertEqual(
            test_document_file_signed.checksum,
            test_document_file_signed_download_file_checksum
        )

    def test_document_no_signature(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        self.assertEqual(EmbeddedSignature.objects.count(), 0)

    def test_new_signed_file(self):
        self._test_document_path = TEST_FILE_SMALL_PATH
        self._upload_test_document()

        with open(file=TEST_SIGNED_DOCUMENT_PATH, mode='rb') as file_object:
            self._test_document.files_upload(
                comment='', file_object=file_object
            )

        self.assertEqual(EmbeddedSignature.objects.count(), 1)

        test_document_file = self._test_document.file_latest
        signature = EmbeddedSignature.objects.first()

        self.assertEqual(signature.document_file, test_document_file)
        self.assertEqual(signature.key_id, TEST_KEY_PUBLIC_ID)
